#!/usr/bin/env python

from gevent.queue import Queue

queue = Queue()

def add_task(task_defn):
    return queue.put_nowait(task_defn)

def get_task():
    return queue.get()

def empty():
    return queue.empty()


#!/usr/bin/env python

import datetime
import sys

import gevent

def throttle_hook(response):
    ratelimited = "x-ratelimit-remaining" in response.headers and \
                  "x-ratelimit-reset" in response.headers 

    if ratelimited:
        remaining = int(response.headers["x-ratelimit-remaining"])
        reset = datetime.datetime.utcfromtimestamp(float(response.headers["x-ratelimit-reset"]))
        now = datetime.datetime.utcnow()
        
        time_to_reset = reset - now
        time_to_sleep = time_to_reset.seconds / remaining

        sys.stderr.write("Throttling... Sleeping for %d secs...\n" % time_to_sleep)
        gevent.sleep(time_to_sleep)



#!/usr/bin/env python

import requests

from gevent import Greenlet

from oauth_hook import OAuthHook

from throttle import throttle_hook
from tasks import add_task, get_task, empty

class Worker(Greenlet):

    def __init__(self, access_token=None, access_token_secret=None, consumer_key=None, consumer_secret=None, header_auth=None):
        Greenlet.__init__(self)

        self.access_token = access_token
        self.access_token_secret = access_token_secret
        self.consumer_key = consumer_key
        self.consumer_secret = consumer_secret
        self.header_auth = header_auth

        oauth_hook = OAuthHook(access_token=access_token, access_token_secret=access_token_secret, 
                               consumer_key=consumer_key, consumer_secret=consumer_secret, 
                               header_auth=header_auth)

        self.client = requests.session(hooks={'pre_request': oauth_hook, 'response': throttle_hook})

    def _run(self):
        while not empty():
            url, kwargs, callback = get_task()

            response = self.client.get(url, **kwargs)
            
            if callable(callback):
                callback(response)

            # Need to add additional requests here, e.g. to loop over 'cursors'
            # ...

    def __str__(self):
        return 'Worker(%s, %s, %s, %s, %s)' % (self.access_token, self.access_token_secret, self.consumer_key, self.consumer_secret, self.header_auth)



#!/usr/bin/env python

import gevent

from tasks import add_task, get_task, empty
from worker import Worker

workers = []

def add_request(url, kwargs, callback=None):
    add_task((url, kwargs, callback))

def add_worker(access_token=None, access_token_secret=None, consumer_key=None, consumer_secret=None, header_auth=None):
    worker = Worker(access_token=access_token, access_token_secret=access_token_secret, consumer_key=consumer_key, consumer_secret=consumer_secret, header_auth=header_auth)
    worker.start()
    workers.append(worker)

def go():
    gevent.joinall(workers)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup, find_packages

version = '0.1.3'

setup(
    name='twitterspawn',
    version=version,
    description='Scalable, concurrent requests to the Twitter REST API, that respect Twitter\'s rate limits, using gevent and requests.',
    long_description=open('README.md').read(),
    author='Steve Winton',
    author_email='stevewinton@gmail.com',
    url='https://github.com/swinton/twitterspawn',
    packages=find_packages(),
    install_requires=['gevent==0.13.7', 'greenlet==0.4.0', 'requests-oauth==0.4.1', 'requests==0.13.7'],
    license='MIT',
    classifiers=(
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Programming Language :: Python',
        "License :: OSI Approved :: BSD License",
        "Operating System :: OS Independent",
        "Topic :: Internet :: WWW/HTTP",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ),
    keywords=['requests', 'python-requests', 'gevent', 'twitterspawn'],
    zip_safe=False,
)